@@ -186,6 +186,17 @@ bool KIdentityProxyModel::dropMimeData(const QMimeData* data, Qt::DropAction act
     return sourceModel()->dropMimeData(data, action, row, column, mapToSource(parent));
 }
 
+/*!
+    \\reimp
+    \since 4.14.7
+ */
+QVariant KIdentityProxyModel::headerData(int section, Qt::Orientation orientation, int role) const
+{
+    if (!sourceModel())
+        return QVariant();
+    return sourceModel()->headerData(section, orientation, role);
+}
+
 /*!
     \reimp
  */
@@ -194,11 +205,8 @@ QModelIndex KIdentityProxyModel::index(int row, int column, const QModelIndex& p
     if (!sourceModel())
         return QModelIndex();
     Q_ASSERT(parent.isValid() ? parent.model() == this : true);
-    if (row < 0 || column < 0 || !hasIndex(row, column, parent))
-        return QModelIndex();
     const QModelIndex sourceParent = mapToSource(parent);
     const QModelIndex sourceIndex = sourceModel()->index(row, column, sourceParent);
-    Q_ASSERT(sourceIndex.isValid());
     return mapFromSource(sourceIndex);
 }
 
