@@ -411,7 +411,10 @@ void KRichTextWidget::createActions(KActionCollection *actionCollection)
         << i18nc("@item:inmenu square list style", "Square")
         << i18nc("@item:inmenu numbered lists", "123")
         << i18nc("@item:inmenu lowercase abc lists", "abc")
-        << i18nc("@item:inmenu uppercase abc lists", "ABC");
+        << i18nc("@item:inmenu uppercase abc lists", "ABC")
+        << i18nc("@item:inmenu lower case roman numerals", "i ii iii")
+        << i18nc("@item:inmenu upper case roman numerals", "I II III");
+
         d->action_list_style->setItems(listStyles);
         d->action_list_style->setCurrentItem(0);
         d->richTextActionList.append((d->action_list_style));
@@ -517,7 +520,7 @@ void KRichTextWidget::createActions(KActionCollection *actionCollection)
         actionCollection->removeAction(d->action_text_superscript);
         d->action_text_superscript = 0;
     }
-    
+
 
     disconnect(this, SIGNAL(currentCharFormatChanged(QTextCharFormat)),
                this, SLOT(_k_updateCharFormatActions(QTextCharFormat)));
@@ -618,7 +621,7 @@ void KRichTextWidget::Private::_k_updateMiscActions()
 
     if ( richTextSupport & SupportDedentLists ) {
         if ( richTextEnabled ) {
-            action_list_dedent->setEnabled( q->canDedentList() );   
+            action_list_dedent->setEnabled( q->canDedentList() );
         } else {
             action_list_dedent->setEnabled( false );
         }
@@ -635,11 +638,11 @@ void KRichTextWidget::Private::_k_setTextForegroundColor()
 {
     QColor currentTextForegroundColor = q->textColor();
 
-    int result = KColorDialog::getColor(currentTextForegroundColor, KColorScheme(QPalette::Active, KColorScheme::View).foreground().color() , q);
-    if (!currentTextForegroundColor.isValid())
-        currentTextForegroundColor = KColorScheme(QPalette::Active, KColorScheme::View).foreground().color() ;
+    const int result = KColorDialog::getColor(currentTextForegroundColor, KColorScheme(QPalette::Active, KColorScheme::View).foreground().color() , q);
     if (result != QDialog::Accepted)
         return;
+    if (!currentTextForegroundColor.isValid())
+        currentTextForegroundColor = KColorScheme(QPalette::Active, KColorScheme::View).foreground().color() ;
 
     q->setTextForegroundColor(currentTextForegroundColor);
 
@@ -650,11 +653,11 @@ void KRichTextWidget::Private::_k_setTextBackgroundColor()
     QTextCharFormat fmt = q->textCursor().charFormat();
     QColor currentTextBackgroundColor = fmt.background().color();
 
-    int result = KColorDialog::getColor(currentTextBackgroundColor, KColorScheme(QPalette::Active, KColorScheme::View).foreground().color() , q);
-    if (!currentTextBackgroundColor.isValid())
-        currentTextBackgroundColor = KColorScheme(QPalette::Active, KColorScheme::View).foreground().color() ;
+    const int result = KColorDialog::getColor(currentTextBackgroundColor, KColorScheme(QPalette::Active, KColorScheme::View).foreground().color() , q);
     if (result != QDialog::Accepted)
         return;
+    if (!currentTextBackgroundColor.isValid())
+        currentTextBackgroundColor = KColorScheme(QPalette::Active, KColorScheme::View).foreground().color() ;
 
     q->setTextBackgroundColor(currentTextBackgroundColor);
 
@@ -681,7 +684,9 @@ void KRichTextWidget::mouseReleaseEvent(QMouseEvent *event)
         // If the painter is active, paint the selection with the
         // correct format.
         if (textCursor().hasSelection()) {
-            textCursor().setCharFormat(d->painterFormat);
+            QTextCursor c = textCursor();
+            c.setCharFormat(d->painterFormat);
+            setTextCursor(c);
         }
         d->painterActive = false;
         d->action_format_painter->setChecked(false);
