@@ -274,22 +274,26 @@ JSValue *DOMCSSStyleDeclarationProtoFunc::callAsFunction(ExecState *exec, JSObje
   KJS_CHECK_THIS( KJS::DOMCSSStyleDeclaration, thisObj );
   CSSStyleDeclarationImpl& styleDecl = *static_cast<DOMCSSStyleDeclaration*>(thisObj)->impl();
 
-  DOM::DOMString s = args[0]->toString(exec).domString();
+  const DOM::DOMString cssProp = args[0]->toString(exec).domString();
 
   switch (id) {
     case DOMCSSStyleDeclaration::GetPropertyValue:
-      return jsString(styleDecl.getPropertyValue(s));
+      return jsString(styleDecl.getPropertyValue(cssProp));
     case DOMCSSStyleDeclaration::GetPropertyCSSValue:
-      return getDOMCSSValue(exec,styleDecl.getPropertyCSSValue(s));
+      return getDOMCSSValue(exec,styleDecl.getPropertyCSSValue(cssProp));
     case DOMCSSStyleDeclaration::RemoveProperty:
-      return jsString(styleDecl.removeProperty(s));
+      return jsString(styleDecl.removeProperty(cssProp));
     case DOMCSSStyleDeclaration::GetPropertyPriority:
-      return jsString(styleDecl.getPropertyPriority(s));
-    case DOMCSSStyleDeclaration::SetProperty:
-      styleDecl.setProperty(args[0]->toString(exec).domString(),
-                            args[1]->toString(exec).domString(),
-                            args[2]->toString(exec).domString());
+      return jsString(styleDecl.getPropertyPriority(cssProp));
+    case DOMCSSStyleDeclaration::SetProperty: {
+      const DOM::DOMString cssVal = args[1]->toString(exec).domString();
+      if (cssVal.isEmpty()) {
+          styleDecl.removeProperty(cssProp);
+      } else {
+          styleDecl.setProperty(cssProp, cssVal, args[2]->toString(exec).domString());
+      }
       return jsUndefined();
+    }
     case DOMCSSStyleDeclaration::Item:
       return jsString(styleDecl.item(args[0]->toInteger(exec)));
     default:
@@ -1202,12 +1206,14 @@ JSValue *DOMCSSPrimitiveValueProtoFunc::callAsFunction(ExecState *exec, JSObject
 const ClassInfo CSSPrimitiveValueConstructor::info = { "CSSPrimitiveValueConstructor", 0, &CSSPrimitiveValueConstructorTable, 0 };
 
 /*
-@begin CSSPrimitiveValueConstructorTable 27
+@begin CSSPrimitiveValueConstructorTable 28
   CSS_UNKNOWN   	DOM::CSSPrimitiveValue::CSS_UNKNOWN	DontDelete|ReadOnly
   CSS_NUMBER    	DOM::CSSPrimitiveValue::CSS_NUMBER	DontDelete|ReadOnly
   CSS_PERCENTAGE	DOM::CSSPrimitiveValue::CSS_PERCENTAGE	DontDelete|ReadOnly
   CSS_EMS       	DOM::CSSPrimitiveValue::CSS_EMS		DontDelete|ReadOnly
   CSS_EXS       	DOM::CSSPrimitiveValue::CSS_EXS		DontDelete|ReadOnly
+  CSS_CHS               DOM::CSSPrimitiveValue::CSS_CHS         DontDelete|ReadOnly
+  CSS_REMS              DOM::CSSPrimitiveValue::CSS_REMS        DontDelete|ReadOnly
   CSS_PX        	DOM::CSSPrimitiveValue::CSS_PX		DontDelete|ReadOnly
   CSS_CM        	DOM::CSSPrimitiveValue::CSS_CM		DontDelete|ReadOnly
   CSS_MM        	DOM::CSSPrimitiveValue::CSS_MM		DontDelete|ReadOnly
