@@ -36,7 +36,6 @@
 #include "css/css_stylesheetimpl.h"
 #include "css/cssproperties.h"
 #include "css/cssvalues.h"
-#include "css/csshelper.h"
 #include "misc/loader.h"
 #include "dom/dom_string.h"
 #include "dom/dom_doc.h"
@@ -72,7 +71,7 @@ void HTMLBodyElementImpl::parseAttribute(AttributeImpl *attr)
 
     case ATTR_BACKGROUND:
     {
-        QString url = khtml::parseURL( attr->value() ).string();
+        QString url = attr->value().parsedUrl().string();
         if (!url.isEmpty()) {
             url = document()->completeURL( url );
             addCSSProperty(CSS_PROP_BACKGROUND_IMAGE, DOMString("url('"+url+"')") );
@@ -177,6 +176,10 @@ void HTMLBodyElementImpl::parseAttribute(AttributeImpl *attr)
         document()->setHTMLWindowEventListener(EventImpl::MESSAGE_EVENT,
             document()->createHTMLEventListener(attr->value().string(), "onmessage", NULL));
         break;        
+    case ATTR_ONHASHCHANGE:
+        document()->setHTMLWindowEventListener(EventImpl::HASHCHANGE_EVENT,
+            document()->createHTMLEventListener(attr->value().string(), "onhashchange", NULL));
+        break;
     case ATTR_NOSAVE:
 	break;
     default:
@@ -250,7 +253,7 @@ HTMLFrameElementImpl::HTMLFrameElementImpl(DocumentImpl *doc)
     marginHeight = -1;
     scrolling = Qt::ScrollBarAsNeeded;
     noresize = false;
-    url = "about:blank";
+    url = QLatin1String("about:blank");;
 }
 
 HTMLFrameElementImpl::~HTMLFrameElementImpl()
@@ -322,7 +325,7 @@ void HTMLFrameElementImpl::parseAttribute(AttributeImpl *attr)
     switch(attr->id())
     {
     case ATTR_SRC:
-        setLocation(khtml::parseURL(attr->val()));
+        setLocation(attr->value().parsedUrl().string());
         break;
     case ATTR_FRAMEBORDER:
     {
@@ -387,7 +390,7 @@ void HTMLFrameElementImpl::attach()
     }
 
     if (parentNode()->renderer() && parentNode()->renderer()->childAllowed()
-                                 && document()->isURLAllowed(url.string()))  {
+                                 && document()->isURLAllowed(url))  {
         RenderStyle* _style = document()->styleSelector()->styleForElement(this);
         _style->ref();
         if ( _style->display() != NONE ) {
@@ -419,7 +422,7 @@ void HTMLFrameElementImpl::computeContent()
         return;
 
     // Bail out on any disallowed URLs
-    if( !document()->isURLAllowed(url.string()) )
+    if( !document()->isURLAllowed(url) )
         return;
 
     // If we have a part already, make sure it's in the right spot...
@@ -434,20 +437,20 @@ void HTMLFrameElementImpl::computeContent()
 
     // Go ahead and load a part... We don't need to clear the widget here,
     // since the -frames- have their lifetime managed, using the name uniqueness.
-    parentPart->loadFrameElement( this, url.string(), name.string() );
+    parentPart->loadFrameElement( this, url, name.string() );
 }
 
-void HTMLFrameElementImpl::setLocation( const DOMString& str )
+void HTMLFrameElementImpl::setLocation( const QString& str )
 {
     url = str;
 
-    if( !document()->isURLAllowed(url.string()) )
+    if( !document()->isURLAllowed(url) )
         return;
 
     // if we already have a child part, ask it to go there..
     KHTMLPart* childPart = contentPart();
     if ( childPart )
-        childPart->openUrl( KUrl( document()->completeURL( url.string() ) ) );
+        childPart->openUrl( KUrl( document()->completeURL( url ) ) );
     else
         setNeedComputeContent(); // otherwise, request it.. 
 }
@@ -752,7 +755,7 @@ void HTMLIFrameElementImpl::parseAttribute(AttributeImpl *attr )
         addHTMLAlignment( attr->value() );
         break;
     case ATTR_SRC:
-        url = khtml::parseURL(attr->val());
+        url = attr->value().parsedUrl().string();
         setNeedComputeContent();
         // ### synchronously start the process?
         break;
@@ -802,7 +805,7 @@ void HTMLIFrameElementImpl::attach()
 
     RenderStyle* style = document()->styleSelector()->styleForElement(this);
     style->ref();
-    if (document()->isURLAllowed(url.string()) && parentNode()->renderer()
+    if (document()->isURLAllowed(url) && parentNode()->renderer()
         && parentNode()->renderer()->childAllowed() && style->display() != NONE) {
         m_render = new (document()->renderArena()) RenderPartObject(this);
         m_render->setStyle(style);
@@ -823,7 +826,7 @@ void HTMLIFrameElementImpl::computeContent()
     if (!parentPart)
         return;
 
-    if (!document()->isURLAllowed(url.string()))
+    if (!document()->isURLAllowed(url))
         return;
 
     if (!inDocument()) {
@@ -835,10 +838,9 @@ void HTMLIFrameElementImpl::computeContent()
     ensureUniqueName();
 
     // make sure "" is handled as about: blank
-    const QString aboutBlank = QLatin1String("about:blank");
-    QString effectiveURL = url.string();
+    QString effectiveURL = url;
     if (effectiveURL.isEmpty())
-        effectiveURL = aboutBlank;
+        effectiveURL = QLatin1String("about:blank");
 
     kDebug(6031) << "-> requesting:" << name.string() << effectiveURL << contentPart();
     
