@@ -36,7 +36,6 @@
 #include "css/cssstyleselector.h"
 #include "css/cssproperties.h"
 #include "css/cssvalues.h"
-#include "css/csshelper.h"
 #include "xml/dom_textimpl.h"
 #include "xml/dom_docimpl.h"
 #include "xml/dom2_eventsimpl.h"
@@ -680,19 +679,18 @@ void HTMLFormElementImpl::submit(  )
         }
 #endif // KHTML_NO_WALLET
 
-        DOMString value = getAttribute(ATTR_ACTION);
-        DOMString url = khtml::parseURL(value);
+        QString url = getAttribute(ATTR_ACTION).parsedUrl().string();
         // ignore base url if 'action' attribute is empty.
-        if (value.isEmpty())
+        if (url.isEmpty())
             url = formUrl.url();
         if(m_post) {
-            view->part()->submitForm( "post", url.string(), form_data,
+            view->part()->submitForm( "post", url, form_data,
                                       m_target.string(),
                                       enctype().string(),
                                       m_boundary );
         }
         else {
-            view->part()->submitForm( "get", url.string(), form_data,
+            view->part()->submitForm( "get", url, form_data,
                                       m_target.string() );
         }
     }
@@ -2449,11 +2447,11 @@ void HTMLSelectElementImpl::parseAttribute(AttributeImpl *attr)
     switch(attr->id())
     {
     case ATTR_SIZE:
-        m_size = qMax( attr->val()->toInt(), 1 );
+        m_size = qMax(attr->value().toInt(), 1);
         setChanged();
         break;
     case ATTR_WIDTH:
-        m_minwidth = qMax( attr->val()->toInt(), 0 );
+        m_minwidth = qMax(attr->value().toInt(), 0);
         break;
     case ATTR_MULTIPLE:
         m_multiple = (attr->val() != 0);
@@ -2520,10 +2518,7 @@ bool HTMLSelectElementImpl::encoding(const QTextCodec* codec, khtml::encodingLis
         (items[0]->id() == ID_OPTION && !items[0]->disabled()) ) {
         HTMLOptionElementImpl* const option = static_cast<HTMLOptionElementImpl*>(items[0]);
         encoded_values += enc_name;
-        if (option->value().isNull())
-            encoded_values += fixUpfromUnicode(codec, option->text().string());
-        else
-            encoded_values += fixUpfromUnicode(codec, option->value().string());
+        encoded_values += fixUpfromUnicode(codec, option->value().string());
         successful = true;
     }
 
@@ -2803,7 +2798,7 @@ DOMString HTMLOptionElementImpl::value() const
     if ( !m_value.isNull() )
         return m_value;
     // Use the text if the value wasn't set.
-    return text().string();
+    return text();
 }
 
 void HTMLOptionElementImpl::setValue(DOMStringImpl* value)
