@@ -40,7 +40,6 @@
 
 #include "xml/dom_docimpl.h"
 #include "css/cssstyleselector.h"
-#include "css/csshelper.h"
 #include "css/cssproperties.h"
 #include "css/cssvalues.h"
 #include "rendering/render_frames.h"
@@ -696,7 +695,7 @@ void HTMLEmbedElementImpl::parseAttribute(AttributeImpl *attr)
   {
      case ATTR_CODE:
      case ATTR_SRC:
-         url = khtml::parseURL(attr->val()).string();
+         url = attr->value().parsedUrl().string();
          setNeedComputeContent();
          break;
      case ATTR_BORDER:
@@ -776,7 +775,7 @@ void HTMLObjectElementImpl::parseAttribute(AttributeImpl *attr)
   switch ( attr->id() )
   {
     case ATTR_DATA:
-      url = khtml::parseURL( attr->val() ).string();
+      url = attr->value().parsedUrl().string();
       setNeedComputeContent();
       break;
     case ATTR_CLASSID:
