@@ -36,9 +36,6 @@
 #  endif
 #endif
 
-#include <khtml_global.h>
-#include <khtml_settings.h>
-
 #include <kdebug.h>
 #include <kglobal.h>
 
@@ -443,6 +440,8 @@ struct CachedFontFamilyKey // basically, FontDef minus the size (and for now SC)
 
     CachedFontFamilyKey() {}
 
+    // resolvedFamily is not actually what it claims to be, as it comes from css style.
+    // e.g. it could be something like "foo,Open Sans,Impact"
     CachedFontFamilyKey(const QString& resolvedFamily, int weight, bool italic) :
             family(resolvedFamily), weight(weight), italic(italic)
     {}
@@ -516,16 +515,18 @@ CachedFontFamily* Font::queryFamily(const QString& name, int weight, bool italic
 	QFont font(name);
 	font.setItalic(italic);
 	font.setWeight(weight);
-
+        // Use resolved family name to query font database
+        const QFontInfo fontInfo(font);
 	QFontDatabase db;
-	QString styleString = db.styleString(font);
+        const QString resolvedFamily = fontInfo.family();
+        const QString styleString = db.styleString(fontInfo);
 	f = new CachedFontFamily;
 	f->def       = key;
-	f->scaleable = db.isSmoothlyScalable(font.family(), styleString);
+	f->scaleable = db.isSmoothlyScalable(resolvedFamily, styleString);
 
 	if (!f->scaleable) {
 	    /* Cache size info */
-	    f->sizes = db.smoothSizes(font.family(), styleString);
+	    f->sizes = db.smoothSizes(resolvedFamily, styleString);
 	}
 
 	fontCache->insert(key, f);
@@ -579,6 +580,14 @@ CachedFontInstance::CachedFontInstance(CachedFontFamily* p, int sz):
     descent = fm.descent();
     height  = fm.height();
     lineSpacing = fm.lineSpacing();
+    xHeight = fm.xHeight();
+
+    const QChar zeroChar((ushort)48);
+    if (!fm.inFont(zeroChar)) {
+        m_zeroCharWidth = -1;
+    } else {
+        m_zeroCharWidth = (int)cachedCharWidth(zeroChar);
+    }
 }
 
 void CachedFontInstance::invalidate()
@@ -601,6 +610,14 @@ void CachedFontInstance::invalidate()
     descent = fm.descent();
     height  = fm.height();
     lineSpacing = fm.lineSpacing();
+    xHeight = fm.xHeight();
+
+    const QChar zeroChar((ushort)48);
+    if (!fm.inFont(zeroChar)) {
+        m_zeroCharWidth = -1;
+    } else {
+        m_zeroCharWidth = (int)cachedCharWidth(zeroChar);
+    }
 }
 
 CachedFontInstance::~CachedFontInstance()
@@ -623,11 +640,9 @@ unsigned CachedFontInstance::calcAndCacheWidth(unsigned short codePoint)
 
 void Font::update(int logicalDpiY) const
 {
-    QString familyName = fontDef.family.isEmpty() ? KHTMLGlobal::defaultHTMLSettings()->stdFontName() : fontDef.family;
-    CachedFontFamily* family = queryFamily(familyName, fontDef.weight, fontDef.italic);
+    CachedFontFamily* family = queryFamily(fontDef.family, fontDef.weight, fontDef.italic);
 
     int size = fontDef.size;
-    const int lDpiY = qMax(logicalDpiY, 96);
 
     // ok, now some magic to get a nice unscaled font
     // all other font properties should be set before this one!!!!
@@ -637,6 +652,7 @@ void Font::update(int logicalDpiY) const
         // lets see if we find a nice looking font, which is not too far away
         // from the requested one.
         // kDebug(6080) << "khtml::setFontSize family = " << f.family() << " size requested=" << size;
+        const float toPix = qMax(logicalDpiY, 96) / 72.0f;
 
         float diff = 1; // ### 100% deviation
         float bestSize = 0;
@@ -646,7 +662,7 @@ void Font::update(int logicalDpiY) const
 
         for( ; it != itEnd; ++it )
         {
-            float newDiff = ((*it)*(lDpiY/72.) - float(size))/float(size);
+            float newDiff = (((*it) * toPix) - float(size)) / float(size);
             //kDebug( 6080 ) << "smooth font size: " << *it << " diff=" << newDiff;
             if(newDiff < 0) newDiff = -newDiff;
             if(newDiff < diff)
@@ -657,7 +673,7 @@ void Font::update(int logicalDpiY) const
         }
         //kDebug( 6080 ) << "best smooth font size: " << bestSize << " diff=" << diff;
         if ( bestSize != 0 && diff < 0.2 ) // 20% deviation, otherwise we use a scaled font...
-            size = (int)((bestSize*lDpiY) / 72);
+            size = static_cast<int>(bestSize * toPix);
     }
 
     // make sure we don't bust up X11
@@ -692,7 +708,7 @@ void Font::initDefault()
     // ### this may cache an instance we don't need though; but font family
     // is extremely likely to be used
     Font f;
-    f.fontDef.size = 10;
+    f.fontDef.size = 16;
     f.update(96);
 
     defaultCFI = f.cfi.get();
@@ -740,4 +756,4 @@ float Font::floatWidth(QChar* str, int pos, int len) const
 
 }
 
-// kate: indent-width 4; replace-tabs off; tab-width 8; space-indent on; hl c++;
+// kate: indent-width 4; replace-tabs on; tab-width 8; space-indent on; hl c++;
