@@ -91,20 +91,18 @@ using namespace DOM;
         {
             noBorder = false;
             left = right = top = bottom = 0;
-            m_proxy = qobject_cast<KdeUiProxyStyle*>(parent->style());
+            clearButtonOverlay = 0;
             setParent(parent);
         }
 
-        QStyle* proxy() const { return m_proxy ? m_proxy : style(); }
-
         QRect subElementRect(SubElement element, const QStyleOption *option, const QWidget *widget) const
         {
-            QRect r = proxy()->subElementRect(element, option, widget);
+            QRect r = style()->subElementRect(element, option, widget);
             switch (element) {
               case QStyle::SE_PushButtonContents:
               case QStyle::SE_LineEditContents:
               case QStyle::SE_ShapedFrameContents:
-                r.adjust(left, top, -right, -bottom);
+                r.adjust(left, top, -qMax(0, right - clearButtonOverlay), -bottom);
               default:
                 break;
             }
@@ -113,16 +111,71 @@ using namespace DOM;
 
         void drawControl(ControlElement element, const QStyleOption *option, QPainter *painter, const QWidget *widget) const
         {
-            if ( noBorder && element == QStyle::CE_PushButton ) {
-               const QStyleOptionButton *o = qstyleoption_cast<const QStyleOptionButton *>(option);
-               if (o) {
-                   QStyleOptionButton opt = *o;
-                   opt.rect = proxy()->subElementRect(SE_PushButtonFocusRect, &opt, widget);
-                   KdeUiProxyStyle::drawControl(CE_PushButtonLabel, &opt, painter, widget);
-               }
-               return;
+            if (element == QStyle::CE_ComboBoxLabel) {
+                const QStyleOptionComboBox *o = qstyleoption_cast<const QStyleOptionComboBox*>(option);
+                if (o) {
+                    QStyleOptionComboBox comboOpt = *o;
+                    comboOpt.currentText = comboOpt.currentText.trimmed();
+                    // by default combobox label is drawn left justified, vertical centered
+                    // translate it to reflect padding values
+                    comboOpt.rect.translate(left, (top - bottom) / 2);
+                    if (noBorder) {
+                        // Need to expand a bit for some styles
+                        comboOpt.rect.adjust(-1, -2, 1, 2);
+                        comboOpt.state &= ~State_On;
+                    }
+                    return style()->drawControl(element, &comboOpt, painter, widget);
+                }
             }
-            KdeUiProxyStyle::drawControl(element,option,painter,widget);
+
+            style()->drawControl(element, option, painter, widget);
+        }
+
+        void drawComplexControl(ComplexControl cc, const QStyleOptionComplex *opt, QPainter *painter, const QWidget *widget) const
+        {
+            if ((cc == QStyle::CC_ComboBox) && noBorder) {
+                if (const QStyleOptionComboBox *cbOpt = qstyleoption_cast<const QStyleOptionComboBox*>(opt)) {
+                    bool enabled = (cbOpt->state & State_Enabled);
+                    QColor color = cbOpt->palette.color(QPalette::ButtonText);
+                    painter->save();
+                    painter->setBackgroundMode(Qt::TransparentMode);
+                    painter->setPen(color);
+                    painter->setRenderHint(QPainter::Antialiasing);
+                    // Drop down indicator
+                    QRect arrowRect = style()->subControlRect(cc, opt, SC_ComboBoxArrow, widget);
+                    arrowRect.setTop(cbOpt->rect.top());
+                    arrowRect.setBottom(cbOpt->rect.bottom());
+                    arrowRect.setRight(cbOpt->rect.right() - 1);
+                    if (enabled && (cbOpt->state & State_On))
+                        arrowRect.translate(1, 1); // push effect
+                    //if (!enabled) color = color.lighter();
+                    painter->setBrush(enabled ? QBrush(color, Qt::SolidPattern) : Qt::NoBrush);
+                    QPolygon cbArrowDown;
+                    cbArrowDown.setPoints(6,  3,-2, 4,-2, 0,2, -4,-2, -3,-2, 0,1);
+                    cbArrowDown.translate((arrowRect.x() + (arrowRect.width() >> 1)), (arrowRect.y() + (arrowRect.height() >> 1)));
+                    painter->drawPolygon(cbArrowDown);
+                    // Focus rect (from qcleanlooksstyle)
+                    if (enabled && (cbOpt->state & State_HasFocus)) {
+                        QRect focusRect = style()->subElementRect(SE_ComboBoxFocusRect, cbOpt, widget);
+                        focusRect.adjust(0, -2, 0, 2);
+                        painter->setBrush(QBrush(color, Qt::Dense4Pattern));
+                        painter->setBrushOrigin(focusRect.topLeft());
+                        painter->setPen(Qt::NoPen);
+                        const QRect rects[4] = {
+                            QRect(focusRect.left(), focusRect.top(), focusRect.width(), 1),    // Top
+                            QRect(focusRect.left(), focusRect.bottom(), focusRect.width(), 1), // Bottom
+                            QRect(focusRect.left(), focusRect.top(), 1, focusRect.height()),   // Left
+                            QRect(focusRect.right(), focusRect.top(), 1, focusRect.height())   // Right
+                        };
+                        painter->drawRects(rects, 4);
+                    }
+                    painter->restore();
+
+                    return;
+                }
+            }
+
+            style()->drawComplexControl(cc, opt, painter, widget);
         }
 
         QRect subControlRect(ComplexControl cc, const QStyleOptionComplex* opt, SubControl sc, const QWidget* widget) const
@@ -146,23 +199,23 @@ using namespace DOM;
                     }
 
                     // Now let sizeFromContent add in extra stuff.
-                    maxW = proxy()->sizeFromContents(QStyle::CT_ComboBox, opt, QSize(maxW, 1), widget).width();
+                    maxW = style()->sizeFromContents(QStyle::CT_ComboBox, opt, QSize(maxW, 1), widget).width();
 
                     // How much more room do we need for the text?
                     int extraW = maxW > cbOpt->rect.width() ? maxW - cbOpt->rect.width() : 0;
 
-                    QRect r = proxy()->subControlRect(cc, opt, sc, widget);
+                    QRect r = style()->subControlRect(cc, opt, sc, widget);
                     r.setWidth(r.width() + extraW);
                     return r;
                 }
             }
 
-            return proxy()->subControlRect(cc, opt, sc, widget);
+            return style()->subControlRect(cc, opt, sc, widget);
         }
 
         int left, right, top, bottom;
+        int clearButtonOverlay;
         bool noBorder;
-        KdeUiProxyStyle* m_proxy;
     };
 
 // ---------------------------------------------------------------------
@@ -172,31 +225,27 @@ RenderFormElement::RenderFormElement(HTMLGenericFormElementImpl *element)
 //     , m_state(0)
     , m_proxyStyle(0)
     , m_exposeInternalPadding(false)
+    , m_isOxygenStyle(false)
 {
     // init RenderObject attributes
     setInline(true);   // our object is Inline
-
 }
 
 RenderFormElement::~RenderFormElement()
 {}
 
-void RenderFormElement::setStyle(RenderStyle *style)
+void RenderFormElement::setStyle(RenderStyle *_style)
 {
-    RenderWidget::setStyle(style);
+    RenderWidget::setStyle(_style);
     setPadding();
-}
-
-void RenderFormElement::calcMinMaxWidth()
-{
-    // Some form widgets apply the padding internally (i.e. as if they were
-    // some kind of inline-block). Thus we only want to expose that padding
-    // while layouting (so that width/height calculations are correct), and
-    // then pretend it does not exist, as it is beyond the replaced edge and
-    // thus should not affect other calculations.
-    m_exposeInternalPadding = true;
-    RenderWidget::calcMinMaxWidth();
-    m_exposeInternalPadding = false;
+    if (!shouldDisableNativeBorders()) {
+        // When the widget shows native border, clipping background to border
+        // results in a nasty rendering effects
+        if (style()->backgroundLayers()->backgroundClip() == BGBORDER) {
+            style()->accessBackgroundLayers()->setBackgroundClip(BGPADDING);
+        }
+        m_isOxygenStyle = QApplication::style()->objectName().contains("oxygen");
+    }
 }
 
 int RenderFormElement::paddingTop() const
@@ -273,7 +322,33 @@ void RenderFormElement::updateFromElement()
         document()->quietResetFocus();
 
     RenderWidget::updateFromElement();
-    setPadding();
+}
+
+// Some form widgets apply the padding internally (i.e. as if they were
+// some kind of inline-block). Thus we only want to expose that padding
+// while layouting (so that width/height calculations are correct), and
+// then pretend it does not exist, as it is beyond the replaced edge and
+// thus should not affect other calculations.
+
+void RenderFormElement::calcMinMaxWidth()
+{
+    m_exposeInternalPadding = true;
+    RenderWidget::calcMinMaxWidth();
+    m_exposeInternalPadding = false;
+}
+
+void RenderFormElement::calcWidth()
+{
+    m_exposeInternalPadding = true;
+    RenderWidget::calcWidth();
+    m_exposeInternalPadding = false;
+}
+
+void RenderFormElement::calcHeight()
+{
+    m_exposeInternalPadding = true;
+    RenderWidget::calcHeight();
+    m_exposeInternalPadding = false;
 }
 
 void RenderFormElement::layout()
@@ -283,10 +358,8 @@ void RenderFormElement::layout()
 
     // minimum height
     m_height = 0;
-    m_exposeInternalPadding = true;
     calcWidth();
     calcHeight();
-    m_exposeInternalPadding = false;
 
     if ( m_widget )
         resizeWidget(m_width-borderLeft()-borderRight()-paddingLeft()-paddingRight(),
@@ -295,8 +368,43 @@ void RenderFormElement::layout()
     setNeedsLayout(false);
 }
 
+int RenderFormElement::calcContentWidth(int w) const
+{
+    if (!shouldDisableNativeBorders()) {
+        if (style()->boxSizing() == CONTENT_BOX) {
+            int nativeBorderWidth = m_widget->style()->pixelMetric(QStyle::PM_DefaultFrameWidth, 0, m_widget);
+            return RenderBox::calcContentWidth(w) + 2 * nativeBorderWidth;
+        }
+    }
 
-Qt::AlignmentFlag RenderFormElement::textAlignment() const
+    return RenderBox::calcContentWidth(w);
+}
+
+int RenderFormElement::calcContentHeight(int h) const
+{
+    if (!shouldDisableNativeBorders()) {
+        if (style()->boxSizing() == CONTENT_BOX) {
+            int nativeBorderWidth = m_widget->style()->pixelMetric(QStyle::PM_DefaultFrameWidth, 0, m_widget);
+            return RenderBox::calcContentHeight(h) + 2 * nativeBorderWidth;
+        }
+    }
+
+    return RenderBox::calcContentHeight(h);
+}
+
+void RenderFormElement::paintOneBackground(QPainter *p, const QColor& c, const BackgroundLayer* bgLayer, QRect clipr, int _tx, int _ty, int w, int height)
+{
+    int fudge = 0;
+    if (!shouldDisableNativeBorders()) {
+        fudge = m_isOxygenStyle ? 3 : 1;
+    }
+
+    paintBackgroundExtended(p, c, bgLayer, clipr, _tx, _ty, w, height,
+                                fudge ? fudge : borderLeft() , fudge ? fudge : borderRight(), RenderWidget::paddingLeft(), RenderWidget::paddingRight(),
+                                fudge ? fudge : borderTop(), fudge ? fudge : borderBottom(), RenderWidget::paddingTop(), RenderWidget::paddingBottom());
+}
+
+Qt::Alignment RenderFormElement::textAlignment() const
 {
     switch (style()->textAlign()) {
         case LEFT:
@@ -498,6 +606,8 @@ RenderSubmitButton::RenderSubmitButton(HTMLInputElementImpl *element)
     setQWidget(p);
     //p->setAutoMask(true);
     p->setMouseTracking(true);
+    p->setDefault(false);
+    p->setAutoDefault(false);
 }
 
 static inline void setStyleSheet_helper(const QString& s, QWidget* w)
@@ -603,13 +713,9 @@ void RenderSubmitButton::calcMinMaxWidth()
     s = s.expandedTo(QApplication::globalStrut());
     int margin = pb->style()->pixelMetric( QStyle::PM_ButtonMargin) +
               pb->style()->pixelMetric( QStyle::PM_DefaultFrameWidth ) * 2;
-    int w = ts.width() + margin;
 
+    int w = ts.width() + margin;
     int h = s.height();
-    if (pb->isDefault() || pb->autoDefault()) {
-        int dbw = pb->style()->pixelMetric( QStyle::PM_ButtonDefaultIndicator ) * 2;
-        w += dbw;
-    }
 
     assert(includesPadding());
     int hpadding = RenderWidget::paddingLeft() + RenderWidget::paddingRight();
@@ -964,11 +1070,6 @@ void LineEditWidget::clearHistoryActivated()
       compObj()->clear();
 }
 
-void LineEditWidget::paintEvent( QPaintEvent *pe )
-{
-    KLineEdit::paintEvent( pe );
-}
-
 bool LineEditWidget::event( QEvent *e )
 {
     if (KLineEdit::event(e))
@@ -1042,7 +1143,8 @@ void RenderLineEdit::setStyle(RenderStyle* _style)
 {
     RenderFormElement::setStyle( _style );
 
-    widget()->setAlignment(textAlignment());
+    if (widget()->alignment() != textAlignment())
+        widget()->setAlignment(textAlignment());
 
     bool showClearButton = (!shouldDisableNativeBorders() && !_style->hasBackgroundImage());
 
@@ -1061,6 +1163,10 @@ void RenderLineEdit::setStyle(RenderStyle* _style)
             }
         }
     }
+
+    if (m_proxyStyle) {
+        static_cast<KHTMLProxyStyle*>(m_proxyStyle)->clearButtonOverlay = qMax(0, widget()->clearButtonUsedSize().width());
+    }
 }
 
 void RenderLineEdit::highLightWord( unsigned int length, unsigned int pos )
@@ -1106,14 +1212,14 @@ void RenderLineEdit::calcMinMaxWidth()
     const QFontMetrics &fm = style()->fontMetrics();
     QSize s;
 
-    int size = element()->size();
+    int size = (element()->size() > 0) ? (element()->size() + 1) : 17; // "some"
 
     int h = fm.lineSpacing();
-    int w = fm.width( 'x' ) * (size > 0 ? size+1 : 17); // "some"
+    int w = (fm.height() * size) / 2; // on average a character cell is twice as tall as it is wide
 
     QStyleOptionFrame opt;
     opt.initFrom(widget());
-    if (static_cast<LineEditWidget*>(widget())->hasFrame())
+    if (widget()->hasFrame())
         opt.lineWidth = widget()->style()->pixelMetric(QStyle::PM_DefaultFrameWidth, &opt, widget());
 
     s = QSize(w, qMax(h, 14));
@@ -1422,21 +1528,22 @@ void RenderFileButton::calcMinMaxWidth()
     KHTMLAssert( !minMaxKnown() );
 
     const QFontMetrics &fm = style()->fontMetrics();
-    int size = element()->size();
+    int size = (element()->size() > 0) ? (element()->size() + 1) : 17; // "some"
 
     int h = fm.lineSpacing();
-    int w = fm.width( 'x' ) * (size > 0 ? size+1 : 17); // "some"
-    KLineEdit* edit = static_cast<KUrlRequester*>( m_widget )->lineEdit();
+    int w = (fm.height() * size) / 2; // on average a character cell is twice as tall as it is wide
+    KLineEdit* edit = widget()->lineEdit();
 
     QStyleOptionFrame opt;
     opt.initFrom(edit);
     if (edit->hasFrame())
         opt.lineWidth = edit->style()->pixelMetric(QStyle::PM_DefaultFrameWidth, &opt, edit);
-    QSize s = edit->style()->sizeFromContents(QStyle::CT_LineEdit,
-                                             &opt,
-          QSize(w, qMax(h, 14)), edit)
-        .expandedTo(QApplication::globalStrut());
-    QSize bs = static_cast<KUrlRequester*>( m_widget )->minimumSizeHint() - edit->minimumSizeHint();
+
+    QSize s(w, qMax(h, 14));
+    s = edit->style()->sizeFromContents(QStyle::CT_LineEdit, &opt, s, edit);
+    s = s.expandedTo(QApplication::globalStrut());
+
+    QSize bs = widget()->minimumSizeHint() - edit->minimumSizeHint();
 
     setIntrinsicWidth( s.width() + bs.width() );
     setIntrinsicHeight( qMax(s.height(), bs.height()) );
@@ -1457,8 +1564,8 @@ void RenderFileButton::updateFromElement()
     KLineEdit* edit = widget()->lineEdit();
     bool blocked = edit->blockSignals(true);
     edit->setText(element()->value().string());
-    edit->blockSignals(false);
-    edit->setModified(blocked );
+    edit->blockSignals(blocked);
+    edit->setModified(false);
 
     RenderFormElement::updateFromElement();
 }
@@ -1665,6 +1772,15 @@ void RenderSelect::clearItemFlags(int index, Qt::ItemFlags flags)
     }
 }
 
+void RenderSelect::setStyle(RenderStyle *_style)
+{
+    RenderFormElement::setStyle(_style);
+    if (!m_useListBox) {
+        KHTMLProxyStyle* proxyStyle = static_cast<KHTMLProxyStyle*>(getProxyStyle());
+        proxyStyle->noBorder = shouldDisableNativeBorders();
+    }
+}
+
 void RenderSelect::updateFromElement()
 {
     m_ignoreSelectEvents = true;
@@ -1681,10 +1797,14 @@ void RenderSelect::updateFromElement()
     if (oldMultiple != m_multiple || oldSize != m_size) {
         if (m_useListBox != oldListbox) {
             // type of select has changed
-            if(m_useListBox)
+            if (m_useListBox)
                 setQWidget(createListBox());
             else
                 setQWidget(createComboBox());
+
+            // Call setStyle() to fix unwanted font size change (#142722)
+            // and to update our proxy style properties
+            setStyle(style());
         }
 
         if (m_useListBox && oldMultiple != m_multiple) {
@@ -1736,7 +1856,7 @@ void RenderSelect::updateFromElement()
                 // Prefer label if set
                 DOMString label = optElem->getAttribute(ATTR_LABEL);
                 if (!label.isEmpty())
-                    domText = label.string();
+                    domText = label;
                 domText = domText.implementation()->collapseWhiteSpace(false, false);
 
                 QString   text;
@@ -1762,8 +1882,10 @@ void RenderSelect::updateFromElement()
                 if (disabled)
                     clearItemFlags(listIndex, Qt::ItemIsSelectable | Qt::ItemIsEnabled);
             }
-            else
+            else {
                 KHTMLAssert(false);
+            }
+
             m_selectionChanged = true;
         }
 
@@ -1791,9 +1913,8 @@ short RenderSelect::baselinePosition( bool f ) const
     if (m_useListBox)
         return RenderFormElement::baselinePosition(f);
 
-    bool hasFrame = static_cast<KComboBox*>(widget())->hasFrame();
-    int bTop = hasFrame ? 0 : borderTop();
-    int bBottom = hasFrame ? 0 : borderBottom();
+    int bTop = shouldDisableNativeBorders() ? borderTop() : 0;
+    int bBottom = shouldDisableNativeBorders() ? borderBottom() : 0;
     int ret = (height()-RenderWidget::paddingTop()-RenderWidget::paddingBottom()-bTop-bBottom+1)/2;
     ret += marginTop() + RenderWidget::paddingTop() + bTop;
     ret += ((fontMetrics( f ).ascent())/2)-2;
@@ -1804,7 +1925,6 @@ void RenderSelect::calcMinMaxWidth()
 {
     KHTMLAssert( !minMaxKnown() );
 
-    m_exposeInternalPadding = true;
     if (m_optionsChanged)
         updateFromElement();
 
@@ -1815,7 +1935,6 @@ void RenderSelect::calcMinMaxWidth()
     // ### end FIXME
 
     RenderFormElement::calcMinMaxWidth();
-    m_exposeInternalPadding = false;
 }
 
 void RenderSelect::layout( )
@@ -1886,8 +2005,17 @@ void RenderSelect::layout( )
     }
     else {
         QSize s(m_widget->sizeHint());
-        setIntrinsicWidth( s.width() );
-        setIntrinsicHeight( s.height() );
+        int w = s.width();
+        int h = s.height();
+
+        if (shouldDisableNativeBorders()) {
+            const int dfw = 2 * m_widget->style()->pixelMetric(QStyle::PM_DefaultFrameWidth, 0, m_widget);
+            w -= dfw;
+            h -= dfw;
+        }
+
+        setIntrinsicWidth(w);
+        setIntrinsicHeight(h);
     }
 
     /// uuh, ignore the following line..
@@ -1899,7 +2027,7 @@ void RenderSelect::layout( )
 
     bool foundOption = false;
     for (int i = 0; i < listItems.size() && !foundOption; i++)
-	foundOption = (listItems[i]->id() == ID_OPTION);
+         foundOption = (listItems[i]->id() == ID_OPTION);
 
     m_widget->setEnabled(foundOption && ! element()->disabled());
 }
@@ -1993,8 +2121,7 @@ void RenderSelect::setOptionsChanged(bool _optionsChanged)
 
 void RenderSelect::setPadding()
 {
-    if (m_size > 1 || m_multiple)
-        RenderFormElement::setPadding();
+    RenderFormElement::setPadding();
 }
 
 ListBoxWidget* RenderSelect::createListBox()
@@ -2069,21 +2196,17 @@ TextAreaWidget::TextAreaWidget(int wrap, QWidget* parent)
     KCursor::setAutoHideCursor(viewport(), true);
     setAcceptRichText (false);
     setMouseTracking(true);
-
-}
-
-void TextAreaWidget::scrollContentsBy( int dx, int dy )
-{
-    KTextEdit::scrollContentsBy(dx, dy);
-    update();
-
 }
 
 TextAreaWidget::~TextAreaWidget()
 {
 }
 
-
+void TextAreaWidget::scrollContentsBy( int dx, int dy )
+{
+    KTextEdit::scrollContentsBy(dx, dy);
+    update();
+}
 
 bool TextAreaWidget::event( QEvent *e )
 {
@@ -2133,8 +2256,6 @@ void TextAreaWidget::keyPressEvent(QKeyEvent *e)
 RenderTextArea::RenderTextArea(HTMLTextAreaElementImpl *element)
     : RenderFormElement(element)
 {
-    scrollbarsStyled = false;
-
     TextAreaWidget *edit = new TextAreaWidget(element->wrap(), view());
     setQWidget(edit);
     const KHTMLSettings *settings = view()->part()->settings();
@@ -2144,6 +2265,7 @@ RenderTextArea::RenderTextArea(HTMLTextAreaElementImpl *element)
     connect(edit,SIGNAL(textChanged()),this,SLOT(slotTextChanged()));
 
     setText(element->value().string());
+    m_textAlignment = edit->alignment();
 }
 
 RenderTextArea::~RenderTextArea()
@@ -2204,18 +2326,27 @@ void RenderTextArea::calcMinMaxWidth()
 
 void RenderTextArea::setStyle(RenderStyle* _style)
 {
-    bool unsubmittedFormChange = element()->m_unsubmittedFormChange;
-
     RenderFormElement::setStyle(_style);
 
-    bool blocked = widget()->blockSignals(true);
-    widget()->setAlignment(textAlignment());
-    widget()->blockSignals(blocked);
-
-    scrollbarsStyled = false;
-
-    element()->m_unsubmittedFormChange = unsubmittedFormChange;
     TextAreaWidget* w = static_cast<TextAreaWidget*>(m_widget);
+
+    if (m_textAlignment != textAlignment()) {
+        m_textAlignment = textAlignment();
+        bool unsubmittedFormChange = element()->m_unsubmittedFormChange;
+        bool blocked = w->blockSignals(true);
+        int cx = w->horizontalScrollBar()->value();
+        int cy = w->verticalScrollBar()->value();
+        QTextCursor tc = w->textCursor();
+        // Set alignment on all textarea's paragraphs
+        w->selectAll();
+        w->setAlignment(m_textAlignment);
+        w->setTextCursor(tc);
+        w->horizontalScrollBar()->setValue(cx);
+        w->verticalScrollBar()->setValue(cy);
+        w->blockSignals(blocked);
+        element()->m_unsubmittedFormChange = unsubmittedFormChange;
+    }
+
     if (style()->overflowX() == OSCROLL)
         w->setHorizontalScrollBarPolicy( Qt::ScrollBarAlwaysOn );
     else if (style()->overflowX() == OHIDDEN)
@@ -2230,21 +2361,6 @@ void RenderTextArea::setStyle(RenderStyle* _style)
         w->setVerticalScrollBarPolicy( Qt::ScrollBarAsNeeded );
 }
 
-void RenderTextArea::layout()
-{
-    KHTMLAssert( needsLayout() );
-
-    RenderFormElement::layout();
-
-    TextAreaWidget* w = static_cast<TextAreaWidget*>(m_widget);
-
-    if (!scrollbarsStyled) {
-        w->horizontalScrollBar()->setPalette(style()->palette());
-        w->verticalScrollBar()->setPalette(style()->palette());
-        scrollbarsStyled=true;
-    }
-}
-
 short RenderTextArea::scrollWidth() const
 {
     return RenderObject::scrollWidth();
@@ -2263,33 +2379,31 @@ void RenderTextArea::setText(const QString& newText)
 
     // When this is called, m_value in the element must have just
     // been set to new value --- see if we have any work to do
-    if ( newText != text() ) {
+
+    QString oldText = text();
+    int oldTextLen = oldText.length();
+    int newTextLen = newText.length();
+    if (newTextLen != oldTextLen || newText != oldText) {
         bool blocked = w->blockSignals(true);
-        QTextCursor tc = w->textCursor();
-        bool atEnd = tc.atEnd();
-        bool atStart = tc.atStart();
         int cx = w->horizontalScrollBar()->value();
         int cy = w->verticalScrollBar()->value();
-        QString oldText = w->toPlainText();
+        // Not using setPlaintext as it resets text alignment property
+        int minLen = qMin(newTextLen, oldTextLen);
         int ex = 0;
-        int otl = oldText.length();
-        if (otl && newText.length() > otl) {
-            while (ex < otl && newText[ex] == oldText[ex])
-                ++ex;
-            QTextCursor tc(w->document());
-            tc.setPosition( ex, QTextCursor::MoveAnchor );
-            tc.movePosition(QTextCursor::End, QTextCursor::KeepAnchor);
-            tc.insertText(newText.right( newText.length()-ex ));
-        } else {
-            w->setPlainText( newText );
-        }
+        while (ex < minLen && (newText.at(ex) == oldText.at(ex)))
+               ++ex;
+        QTextCursor tc = w->textCursor();
+        tc.setPosition(ex, QTextCursor::MoveAnchor);
+        tc.movePosition(QTextCursor::End, QTextCursor::KeepAnchor);
+        tc.insertText(newText.right(newTextLen - ex));
+
+        if (oldTextLen == 0)
+            tc.movePosition(QTextCursor::Start, QTextCursor::MoveAnchor);
+        else
+            tc.movePosition(QTextCursor::End, QTextCursor::MoveAnchor);
         w->setTextCursor(tc);
-        if (atEnd)
-           tc.movePosition(QTextCursor::End, QTextCursor::MoveAnchor);
-        else if (atStart)
-           tc.movePosition(QTextCursor::Start, QTextCursor::MoveAnchor);
-        w->horizontalScrollBar()->setValue( cx );
-        w->verticalScrollBar()->setValue( cy );
+        w->horizontalScrollBar()->setValue(cx);
+        w->verticalScrollBar()->setValue(cy);
         w->blockSignals(blocked);
     }
 }
