@@ -37,10 +37,8 @@
 #include <html/htmltokenizer.h>
 #include "dom_restyler.h"
 
-#include <css/csshelper.h>
 #include <css/cssstyleselector.h>
 #include <css/css_stylesheetimpl.h>
-#include <misc/helper.h>
 #include <misc/seed.h>
 #include <misc/loader.h>
 #include <ecma/kjs_proxy.h>
@@ -639,8 +637,12 @@ CommentImpl *DocumentImpl::createComment ( DOMStringImpl* data )
     return new CommentImpl( docPtr(), data );
 }
 
-CDATASectionImpl *DocumentImpl::createCDATASection ( DOMStringImpl* data )
+CDATASectionImpl *DocumentImpl::createCDATASection ( DOMStringImpl* data, int &exceptioncode )
 {
+    if (isHTMLDocument()) {
+        exceptioncode = DOMException::NOT_SUPPORTED_ERR;
+        return 0;
+    }
     return new CDATASectionImpl( docPtr(), data );
 }
 
@@ -649,8 +651,12 @@ ProcessingInstructionImpl *DocumentImpl::createProcessingInstruction ( const DOM
     return new ProcessingInstructionImpl( docPtr(),target,data);
 }
 
-EntityReferenceImpl *DocumentImpl::createEntityReference ( const DOMString &name )
+EntityReferenceImpl *DocumentImpl::createEntityReference ( const DOMString &name, int &exceptioncode )
 {
+    if (isHTMLDocument()) {
+        exceptioncode = DOMException::NOT_SUPPORTED_ERR;
+        return 0;
+    }
     return new EntityReferenceImpl(docPtr(), name.implementation());
 }
 
@@ -702,11 +708,11 @@ NodeImpl *DocumentImpl::importNode(NodeImpl *importedNode, bool deep, int &excep
     }
     else if(importedNode->nodeType() == Node::CDATA_SECTION_NODE)
     {
-	result = createCDATASection(static_cast<CDATASectionImpl*>(importedNode)->string());
+	result = createCDATASection(static_cast<CDATASectionImpl*>(importedNode)->string(), exceptioncode);
 	deep = false;
     }
     else if(importedNode->nodeType() == Node::ENTITY_REFERENCE_NODE)
-	result = createEntityReference(importedNode->nodeName());
+	result = createEntityReference(importedNode->nodeName(), exceptioncode);
     else if(importedNode->nodeType() == Node::PROCESSING_INSTRUCTION_NODE)
     {
 	result = createProcessingInstruction(importedNode->nodeName(), importedNode->nodeValue().implementation());
@@ -1401,25 +1407,14 @@ void DocumentImpl::recalcStyle( StyleChange change )
         RenderStyle* _style = new RenderStyle();
         _style->setDisplay(BLOCK);
         _style->setVisuallyOrdered( visuallyOrdered );
-        // ### make the font stuff _really_ work!!!!
+        // ### make the font stuff _really_ work!!!! (??)
 
-	khtml::FontDef fontDef;
-	QFont f = KGlobalSettings::generalFont();
-	fontDef.family = f.family();
-	fontDef.italic = f.italic();
-	fontDef.weight = f.weight();
-        if (m_view) {
-            const KHTMLSettings *settings = m_view->part()->settings();
-	    QString stdfont = settings->stdFontName();
-	    if ( !stdfont.isEmpty() )
-		fontDef.family = stdfont;
-
-            fontDef.size = m_styleSelector->fontSizes()[3];
-        }
-
-        //kDebug() << "DocumentImpl::attach: setting to charset " << settings->charset();
+        FontDef fontDef = FontDef();
+        // Initial fontDef.size is 0
+        fontDef.size = m_styleSelector->fontSizes()[3];
         _style->setFontDef(fontDef);
-	_style->htmlFont().update( 0 );
+        _style->htmlFont().update(0);
+
         if ( inCompatMode() )
             _style->setHtmlHacks(true); // enable html specific rendering tricks
 
@@ -1701,6 +1696,18 @@ void DocumentImpl::finishParsing (  )
 
 QString DocumentImpl::completeURL(const QString& url) const
 {
+    if (url.startsWith(QLatin1Char('#'))) {
+        const QString ref = KUrl::fromPercentEncoding(url.mid(1).toUtf8());
+        KUrl u = baseURL();
+        if (ref.isEmpty()) {
+            u.setHTMLRef("");
+        }
+        else {
+            u.setHTMLRef(ref);
+        }
+        return u.url();
+    }
+
     return KUrl(baseURL(),url /*,m_decoderMibEnum*/).url();
 }
 
@@ -1984,16 +1991,16 @@ void DocumentImpl::processHttpEquiv(const DOMString &equiv, const DOMString &con
                 v->part()->scheduleRedirection(delay, v->part()->url().url() );
         } else {
             pos++;
-            while(pos < (int)str.length() && str[pos].isSpace()) pos++;
+            while(pos < str.length() && str[pos].isSpace()) pos++;
             str = str.mid(pos);
             if(str.indexOf("url", 0, Qt::CaseInsensitive ) == 0)  str = str.mid(3);
             str = str.trimmed();
             if ( str.length() && str[0] == '=' ) str = str.mid( 1 ).trimmed();
-            while(str.length() &&
-                  (str[str.length()-1] == ';' || str[str.length()-1] == ','))
+            while (str.length() && (str[str.length()-1] == ';' || str[str.length()-1] == ',')) {
                 str.resize(str.length()-1);
-            str = parseURL( DOMString(str) ).string();
-            QString newURL = document()->completeURL( str );
+            }
+            str = DOMString(str).parsedUrl().string();
+            QString newURL = document()->completeURL(str);
             if ( ok )
                 v->part()->scheduleRedirection(delay, newURL,  delay < 2 || newURL == URL().url());
         }
@@ -2053,8 +2060,8 @@ bool DocumentImpl::prepareMouseEvent( bool readonly, int _x, int _y, MouseEvent
             //qDebug("urlnode: %s  (%d)", getTagName(renderInfo.URLElement()->id()).string().toLatin1().constData(), renderInfo.URLElement()->id());
 
             ElementImpl* e =  static_cast<ElementImpl*>(renderInfo.URLElement());
-            DOMString href = khtml::parseURL(e->getAttribute(ATTR_HREF));
-            DOMString target = e->getAttribute(ATTR_TARGET);
+            const DOMString href = e->getAttribute(ATTR_HREF).parsedUrl();
+            const DOMString target = e->getAttribute(ATTR_TARGET);
 
             if (!target.isNull() && !href.isNull()) {
                 ev->target = target;
