@@ -20,6 +20,7 @@
 
 #include "fileundomanager.h"
 #include "fileundomanager_p.h"
+#include "clipboardupdater_p.h"
 #include "fileundomanager_adaptor.h"
 
 #include <kdatetime.h>
@@ -323,6 +324,10 @@ quint64 FileUndoManager::currentCommandSerialNumber() const
 
 void FileUndoManager::undo()
 {
+    if (d->m_commands.isEmpty()) {
+        return;
+    }
+
     // Make a copy of the command to undo before broadcastPop() pops it.
     UndoCommand cmd = d->m_commands.last();
     assert(cmd.m_valid);
@@ -537,6 +542,7 @@ void FileUndoManagerPrivate::stepMovingFiles()
         {
             kDebug(1203) << "file_move" << op.m_dst << op.m_src;
             m_currentJob = KIO::file_move(op.m_dst, op.m_src, -1, KIO::Overwrite | KIO::HideProgressInfo);
+            KIO::ClipboardUpdater::create(m_currentJob, KIO::ClipboardUpdater::UpdateContent);
             m_undoJob->emitMoving(op.m_dst, op.m_src);
         }
 
