@@ -176,12 +176,6 @@ QList<Nepomuk::ResourceData*> Nepomuk::ResourceManagerPrivate::allResourceData()
 }
 
 
-bool Nepomuk::ResourceManagerPrivate::dataCacheFull() const
-{
-    return dataCnt >= 10000;
-}
-
-
 void Nepomuk::ResourceManagerPrivate::cleanupCache( int num )
 {
     QMutexLocker lock( &mutex );
@@ -201,12 +195,8 @@ void Nepomuk::ResourceManagerPrivate::cleanupCache( int num )
 
 bool Nepomuk::ResourceManagerPrivate::shouldBeDeleted( ResourceData * rd ) const
 {
-    //
-    // We delete data objects in one of two cases:
-    // 1. They are not valid and as such not in one of the ResourceManagerPrivate kickoff lists
-    // 2. The cache is already full and we need to clean up
-    //
-    return( !rd->cnt() && ( !rd->isValid() || dataCacheFull() ));
+    // We only delete ResourceData objects if no other Resource is accessing them
+    return !rd->cnt();
 }
 
 
@@ -233,6 +223,9 @@ void Nepomuk::ResourceManagerPrivate::_k_dbusServiceUnregistered( const QString&
     if( serviceName == QLatin1String("org.kde.NepomukStorage") ) {
         kDebug() << "Nepomuk Storage service went down.";
         cleanupCache(-1);
+        if( m_manager->d->mainModel && m_manager->d->mainModel->isValid() ){
+            m_manager->d->mainModel->disconnect();
+        }
         emit m_manager->nepomukSystemStopped();
     }
 }
@@ -490,7 +483,7 @@ QUrl Nepomuk::ResourceManager::generateUniqueUri( const QString& name )
                                                        "UNION "
                                                        "{ graph <%1> { ?s4 ?4 ?o4 . } . } "
                                                        "}")
-                                   .arg( QString::fromAscii( uri.toEncoded() ) ), Soprano::Query::QueryLanguageSparql ).boolValue() ) {
+                                   .arg( QString::fromLatin1( uri.toEncoded() ) ), Soprano::Query::QueryLanguageSparql ).boolValue() ) {
             return uri;
         }
     }
